#!/usr/bin/env node --harmony  --max-old-space-size=4096
'use strict';
// 定义脚手架的文件路径
// process.env.NODE_PATH = __dirname + '/../node_modules/';

const program = require('commander');
const paths = require('../config/paths');

// 定义当前版本
program
  .version(require('../package').version );

// 定义使用方法
program
  .usage('<command>')
  .option('-i, --init [value]', '创建初始项目')
  .option('-dev, --start [value]', '启动本地开发服务')
  .option('-b, --build [value]', '打包本地项目文件到build目录')
  .option('-ls, --list [value]', '列表出当前已存在项目列表');


// ====

program
  .command('init')
  .alias('i')
  .arguments('[projectName]')
  .description('初始化一个项目')
  .action((projectName) => {
    require('../command/init')(projectName);
  });

program
  .command('dev')
  .alias('start')
  .arguments('[projectName] [port]')
  .description('启动本地开发服务')
  .action((projectName, port) => {
    require('../command/dev')(projectName, port);
  });


program
  .command('build')
  .alias('b')
  .arguments('[projectName]')
  .description('build本地项目文件到build目录')
  .action((projectName) => {
    require('../command/build')(projectName);
  });

program
  .command('release')
  .alias('zip')
  .arguments('[projectName]')
  .description('打包本地项目文件到build目录')
  .action((projectName) => {
    require('../command/release')(projectName);
  });

program
  .command('list')
  .alias('ls')
  .description('当前已存在项目列表')
  .action(() => {
    require('../command/list')();
  });


program
  .command('pre')
  .alias('p')
  .description('预览public目录的服务-未完成')
  .action(() => {
    console.log('todo...')
    // require('../command/pre')();
  });


program
  .command('copy')
  .description('copy文件')
  .action((sourcePath, targetPath, cleanTargetPath) => {
    require('../command/copy')(sourcePath, targetPath, cleanTargetPath);
  });


// =====

program.parse(process.argv);

if(!program.args.length){
  program.help()
}