#!/usr/bin/env node --max-old-space-size=4096
'use strict';
const spawn = require('cross-spawn');
const paths = require('../config/paths');
const path = require('path');
const utils = require('../utils');
const fs = require('fs');
const cwd = process.cwd();
const build = require('../dev-scripts/build');
const copy = require('../dev-scripts/dev-utils/copy');

function getNameFromCwd() {
  const _pathArray = cwd.split(path.sep);
  return _pathArray[_pathArray.length - 1];
}


module.exports = (projectName) => {
  return new Promise((resolve, reject) => {

    const appDirectory = projectName ? path.resolve(paths.projectPath, projectName) : cwd;

    if (utils.fsExists(appDirectory)) {
      process.chdir(appDirectory);
      const appPaths = paths.appPath(appDirectory);

      if (utils.fsExists(appPaths.appPackageJson)) {
        const json = require(appPaths.appPackageJson);
        const cmd = utils.isWin32 ? 'w-build' : 'build';
        if (json.scripts && json.scripts[cmd]) {
          // 执行 npm build
          spawn('npm', ['run', cmd],{stdio: 'inherit', cwd: appDirectory});
          resolve();
        } else {
          // 执行 sinx start script
          const type = json.sinxType || 'web'; // 'web' or 'rax'
          process.env.SINX_TYPE = type;
          process.env.PROJECT_PATH = appDirectory;
          process.env.PROJECT_NAME = projectName ? projectName : getNameFromCwd();
          build(type).then(() => {
            console.log('package.json 里 copyFiles 字段，可以追加copy文件。');
            const json = require(appPaths.appPackageJson);
            if (json.copyFiles) {
              copy(json.copyFiles);
            }
            resolve();
          });
        }
      } else {
        console.error(`当前项目${appDirectory}目录下缺少 package.json 文件 `);
        reject();
      }

    } else {
      console.error(`未找到项目目录: ${appDirectory}`);
      reject();
    }
  });

};