#!/usr/bin/env node --max-old-space-size=4096
'use strict';
const spawn = require('cross-spawn');
const paths = require('../config/paths');
const path = require('path');
const utils = require('../utils');
const fs = require('fs');
const start = require('../dev-scripts/start');



module.exports = (projectName, port) => {
  const cwd = process.cwd();
  const appDirectory = projectName ? path.resolve(paths.projectPath, projectName) : cwd;

  if (utils.fsExists(appDirectory)) {
    process.chdir(appDirectory);
    const appPaths = paths.appPath(appDirectory);

    if (utils.fsExists(appPaths.appPackageJson)) {
      const json = require(appPaths.appPackageJson);
      const cmd = utils.isWin32 ? 'w-start' : 'start';
      if (json.scripts && json.scripts[cmd]) {
        // 执行 npm start
        spawn('npm', [cmd],{stdio: 'inherit', cwd: appDirectory})
      } else {
        // 执行 sinx start script
        const type = json.sinxType || 'web'; // 'web' or 'rax'
        process.env.PORT = port;
        process.env.SINX_TYPE = type;
        start(type);
      }
    } else {
      console.error(`当前项目${appDirectory}目录下缺少 package.json 文件 `)
    }

  } else {
    console.error(`未找到项目目录: ${appDirectory}`)
  }

};