const { prompt } = require('inquirer');
const fs = require('fs');
// const { resolve } = require('path');
const chalk = require('chalk');
const download = require('download-git-repo');
const ora = require('ora');
const cwd = process.cwd();
const execSync = require('child_process').execSync;
const spawn = require('cross-spawn');
const path = require('path');
const paths = require('../config/paths');

const tplList = require(`../templates`);
// const { writeFile } = fs;

const question = [
  {
    type: 'list',
    name: 'templateType',
    message: '选择一个模板类型:',
    // paginated: true, // 模板列表过多时可打开分页显示
    choices () {
      return Object.keys(tplList.tpl);
    }
  },
/*  {
    type: 'confirm',
    name: 'createProjectInCurrentDir',
    message: '是否在当前目录创建项目？',
    default: false
  },*/
  {
    type: 'confirm',
    name: 'autoInstallModules',
    message: '创建项目后是否自动安装依赖？',
    default: 'y'
  },
  {
    type: 'confirm',
    name: 'autoStartDevServer',
    message: '是否自动启动服务？',
    when: (answers) => {
      return answers.autoInstallModules === true;
    },
    default: 'y'
  }
];
module.exports = (projectNameArg) => {

  // 初始化目录
  paths.initPath();


  // 若没有填名称
  if (!projectNameArg) {
    question.splice(1, 0, {
      type: 'input',
      name: 'projectName',
      message: '请输入需要创建的项目名称:',
      validate (val) {
        if (val !== '') {
          return true
        }
        return '项目名称不能为空!'
      }
    })
  }

  return prompt(question).then(({ templateType, projectName, createProjectInCurrentDir, autoInstallModules, autoStartDevServer }) => {
    const _projectName = projectName || projectNameArg;
    const basePath = createProjectInCurrentDir ? cwd : paths.projectPath;
    const projectPath = path.resolve(basePath, _projectName);
    console.log('正在检查目录,准备创建项目到: '+ projectPath);
    const spinner = ora('正在下载模板 - ' + templateType);
    const tpl = tplList.tpl[templateType];
    spinner.start();
    if (!fs.existsSync(projectPath)) {
      download(tpl.url, projectPath, { clone: tpl.clone || false }, (err) => {
        spinner.stop();
        if (err) {
          console.warn('模板下载失败：');
          console.warn(tpl.url);
          console.warn('');
          console.warn(err);
          process.exit(1);
        }
        console.log('项目创建成功.');
        process.chdir(projectPath);
        if (autoInstallModules || autoStartDevServer) {
          return install(projectPath).then(() => {
            if (autoStartDevServer) {
              spawn('sinx', ['start'], {stdio: 'inherit', cwd: projectPath})
            }
          });
        } else {
          return false;
        }
      });
    }else {
      console.warn(projectPath + ' 已经存在，请使用别的名字');
    }
  });
};



function shouldUseYarn() {
  try {
    execSync('yarnpkg --version', {stdio: 'ignore'});
    return true;
  } catch (e) {
    return false;
  }
}

/**
 * run npm/yarn install
 * @param directory - the cwd directory 工作目录
 * @param verbose - enable verbose mode 显示详细进度
 * @returns {Promise}
 */
function install(directory, verbose = false) {
  console.log(chalk.white.bold('正在安装依赖:'));

  const pkgManager = shouldUseYarn() ? 'yarn' : 'npm';
  const args = ['install'];
  if (verbose) {
    args.push('--verbose');
  }

  return new Promise((resolve) => {
    if (!directory) {
      console.error('缺少 directory 参数');
      resolve(false);
    }

    const proc = spawn(pkgManager, args, {stdio: 'inherit', cwd: directory});

    proc.on('close', (code) => {
      if (code !== 0) {
        console.error('`' + pkgManager + ' install` failed');
        resolve(false);
      } else {
        resolve(true);
      }
    });
  });
}