'use strict';

const path = require('path');
const fs = require('fs');
const userHome = require('user-home');
const utils = require('../utils');

const sinxBaseDir = utils.isWin32 ? path.resolve('d:\\', 'Sinx') : path.resolve(userHome, 'Sinx');

function initPath() {
  const _needCreateDirs = ['deploy', 'project'];
  if (!utils.fsExists(sinxBaseDir)) {
    fs.mkdirSync(sinxBaseDir);
  }
  _needCreateDirs.forEach(dir => {
    const _p = path.resolve(sinxBaseDir, dir);
    if (!utils.fsExists(_p)) {
      fs.mkdirSync(_p);
    }
  });
}
function appPath(appDirectory = process.cwd()) {
  const resolvePath = (relativePath) => {
    return path.resolve(appDirectory, relativePath);
  };
  return {
    appPath: appDirectory,
    appBuild: resolvePath('build'),
    appPublic: resolvePath('public'),
    // appAssets: resolvePath('assets'),
    appHtml: resolvePath('public/index.html'),
    appIndexJs: resolvePath('src/index.js'),
    appWebpackConfigJs: resolvePath('webpack.config.js'),
    appPackageJson: resolvePath('package.json'),
    appSrc: resolvePath('src'),
    appNodeModules: resolvePath('node_modules')
  };
}

const paths = {
  initPath,
  appPath,
  sinxBasePath: sinxBaseDir,
  projectPath: path.resolve(sinxBaseDir, 'project'),
  deployPath: path.resolve(sinxBaseDir, 'deploy'),
};

module.exports = paths;
