'use strict';
/* eslint no-console: 0 */
const path = require('path');
const colors = require('chalk');
const webpack = require('webpack');
const utils = require('../../utils');
// const merge = require('webpack-merge');

/**
 * Create webpack compiler instance
 *
 * @param  {Object} webpackConfig webpack config
 * @return {compiler}             webpack compiler instance
 *
 * @see http://webpack.github.io/docs/plugins.html#the-compiler-instance
 */
module.exports = webpackConfig => {
  let compiler;
  let config = Object.assign({}, webpackConfig);


  // 使用本地合并配置
  const UPDATE_FILE_NAME = 'webpack.config.update.js';
  const webpackConfigUpdateFile = path.join(process.cwd(), UPDATE_FILE_NAME);

  if (utils.fsExists(webpackConfigUpdateFile)) {
    console.log('');
    console.log(`检测到 ${webpackConfigUpdateFile}`);
    console.log('正在合并配置文件...');
    const updateConfigFn = require(webpackConfigUpdateFile);
    config = updateConfigFn(webpackConfig);
    // console.log('=====', config);
  }

  // 使用本地配置
  const LOCAL_FILE_NAME = 'webpack.config.local.js';
  const webpackConfigLocalFile = path.join(process.cwd(), LOCAL_FILE_NAME);

  if (utils.fsExists(webpackConfigLocalFile)) {
    console.log('');
    console.log(`检测到 ${webpackConfigLocalFile}`);
    console.log('使用本地webpack配置文件，devServer 仍然使用 sinx 内置服务，若需要完全自由配置，可在 package.json 里配置 start 脚本');
    config = require(webpackConfigLocalFile);
  }

  try {
    compiler = webpack(config);
  } catch (err) {
    console.error(colors.red('[ERR]: Failed to compile.'));
    console.log('');
    console.error(err.message || err);
    console.log('');
    process.exit(1);
  }

  compiler.plugin('done', stats => {
    if (stats.hasErrors()) {
      return console.error(
        stats.toString({
          colors: true
        })
      );
    }

    console.log(
      stats.toString({
        colors: true,
        chunks: false,
        assets: true
      })
    );
  });

  compiler.plugin('failed', err => {
    throw err;
  });

  return compiler;
};
