'use strict';
/* eslint no-console: 0 */
process.env.NODE_ENV = 'development';

process.on('unhandledRejection', err => {
  throw err;
});

const colors = require('chalk');
const spawn = require('cross-spawn');
const createWebpackCompiler = require('./dev-utils/create-webpack-compiler');
const extendWebpackServer = require('./dev-utils/create-webpack-server-compiler');
const options = require('./dev-utils/options');
const portDetect = require('./dev-utils/portDetect');
const opn = require("opn");
const utils = require('../utils');

const paths = require('../config/paths');
const cwd = process.cwd();
const appPath = paths.appPath(cwd);
const baseDir = appPath.appPath;
const WebpackDevServer = require(baseDir + '/node_modules/webpack-dev-server');
/**
 * run webpack dev server
 * @param  {String} type of dev server(`web` or `rax`)
 */
function start(type = 'web') {

  const startDevServer = (port) => {
    const webpackConfigDev = require('./webpack.config/' + type + '/dev.js');
    const compiler = createWebpackCompiler(webpackConfigDev);
    const webpackDevServerConfig = require('./webpack.config/' + type + '/devServer.js');
    const webpackDevServerConfigExtend = extendWebpackServer(webpackDevServerConfig);
    const server = new WebpackDevServer(compiler, webpackDevServerConfigExtend);

    const serverUrl = `${options.protocol()}//${options.host()}:${port}/`;

    const openApp = () => {
      if(utils.fsExists(appPath.appPackageJson)) {
        const json = require(appPath.appPackageJson);
        const cmd = 'electron';
        if (json.scripts && json.scripts[cmd]) {
          console.log('   正在启动 electron...');
          // 执行 npm run electron
          spawn('npm', ['run', cmd],{stdio: 'inherit', cwd: cwd});
        } else {
          console.log('   正在启动浏览器...');
          opn(serverUrl, {app: utils.isWin32 ? 'chrome' : 'google chrome'});
        }
      }
    };

    console.log('');
    openApp();

    server.listen(port, err => {
      if (err) {
        console.log(colors.red('[ERR]: Failed to webpack dev server'));
        console.error(err.message || err);
        process.exit(1);
      }
      console.log('');
      console.log('');
      console.log(colors.green('Starting the development server at:'));
      console.log(`    ${colors.underline.white(serverUrl)}`);
      if (type === 'rax') {
        console.log('');
        console.log('    rax mod模块调试,建议绑定HOST后通过以下域名访问:');
        console.log(`    ${colors.underline.white(`http://local.m.taobao.com:${port}/`)}`);
      }
    });
  };


  const detectEnvPort = () => {
    const port = options.port();
    portDetect(port).then((rs) => {
      if (rs) {
        process.env.PORT = parseInt(port, 10);
        startDevServer(port);
      } else {
        process.env.PORT = parseInt(port, 10) + 1;
        detectEnvPort();
      }
    });
  };

  detectEnvPort();

}


module.exports = start;
