'use strict';

/* eslint no-console: 0 */
const address = require('address');
const qrcode = require('qrcode-terminal');

const paths = require('../../../config/paths');
const appPaths = paths.appPath(process.cwd());
const appNodeModules = appPaths.appNodeModules;
const requireNodeModules = (modulePath) => {
  return require(appNodeModules + '/' + modulePath);
};
const requireResolveNodeModules = (modulePath) => {
  return require.resolve(appNodeModules + '/' + modulePath);
};


const webpack = requireNodeModules('webpack');
const webpackConfigBase = require('./base');
const options = require('../../dev-utils/options');
const RaxHotModuleReplacementPlugin = requireNodeModules('rax-hot-module-replacement-webpack-plugin');
const raxHotLoaderPatch = requireResolveNodeModules('rax-hot-loader/patch');

const webpackConfigDev = Object.assign({}, webpackConfigBase);
const ipv4 = address.ip();
const port = options.port();
const webUrl = 'http://' + ipv4 + ':' + port;
const bundleUrl = 'http://' + ipv4 + ':' + port + '/index.bundle.js';
const taobaoBundleUrl = 'http://local.m.taobao.com:' + port + '/index.bundle.js';
const weexBundleUrl = `${bundleUrl}?_wx_tpl=${bundleUrl}`;
const weexTaobaoBundleUrl = `${taobaoBundleUrl}?_wx_tpl=${taobaoBundleUrl}`;

qrcode.generate(webUrl, { small: true });
console.log('Web: scan above QRCode ' + webUrl + ' or direct open in browser.\n');

qrcode.generate(weexBundleUrl, { small: true });
console.log('Weex: scan above QRCode ' + weexBundleUrl + ' use weex playground.\n');

qrcode.generate(weexTaobaoBundleUrl, { small: true });
console.log('Taobao: scan above QRCode ' + weexTaobaoBundleUrl + ' use weex playground.\n');

// enable source map
// webpackConfigDev.devtool = 'inline-module-source-map';
webpackConfigDev.devtool = '#cheap-module-source-map';
webpackConfigDev.entry = {
  'index.bundle': [appPaths.appIndexJs]
};
// Module hot accept
webpackConfigDev.module.loaders.forEach(loader => {
  if (loader.test.toString() === /\.jsx?$/.toString()) {
    loader.loaders.push(`${require.resolve('./helper/module-hot-accept')}?appIndex=${appPaths.appIndexJs}`);
  }
});

Object.keys(webpackConfigDev.entry).forEach(point => {
  // Enable hot reloading
  webpackConfigDev.entry[point].unshift(raxHotLoaderPatch);
  // webpackConfigDev.entry[point].unshift(`${require.resolve('webpack-dev-server/client')}?${options.protocol}//${options.host}:${options.port}`);
  // // bundle the client for webpack-dev-server
  // // and connect to the provided endpoint
  // webpackConfigDev.entry[point].unshift(require.resolve('webpack/hot/only-dev-server'));
  // // bundle the client for hot reloading
  // // only- means to only hot reload for successful updates

  // hot reaload client.
  webpackConfigDev.entry[point].unshift(require.resolve('./helper/webpackHotDevClient'));
});

// Only work on web
webpackConfigDev.plugins.push(new RaxHotModuleReplacementPlugin());
webpackConfigDev.plugins.push(new webpack.NoEmitOnErrorsPlugin());

module.exports = webpackConfigDev;
